@@ -1,4 +1,4 @@
-static pe_ring Timers;
+static pe_ring Timeables;
 
 typedef struct pe_tmevent pe_tmevent;
 struct pe_tmevent {
@@ -8,8 +8,9 @@ struct pe_tmevent {
 
 static void checkTimers()
 {
-  pe_ring *rg = Timers.next;
-  while (rg->self && ((pe_tmevent*)rg->self)->tm.at < SvNVX(NowSV)) {
+  pe_ring *rg = Timeables.next;
+  double now = EvNOW;
+  while (rg->self && ((pe_tmevent*)rg->self)->tm.at < now) {
     pe_ring *nxt = rg->next;
     PE_RING_DETACH(rg);
     EvACTIVE_off(rg->self);
@@ -20,22 +21,33 @@ static void checkTimers()
 
 static double timeTillTimer()
 {
-  pe_ring *rg = Timers.next;
+  pe_ring *rg = Timeables.next;
   if (!rg->self)
     return 3600;
-  pe_cache_now();
-  return ((pe_tmevent*) rg->self)->tm.at - SvNVX(NowSV);
+  return ((pe_tmevent*) rg->self)->tm.at - EvNOW;
+}
+
+static void db_show_timeables()
+{
+  pe_tmevent *ev;
+  ev = (pe_tmevent*) Timeables.next->self;
+  while (ev) {
+    warn("0x%x : %.2f\n", ev, ev->tm.at);
+    ev = (pe_tmevent*) ev->tm.ring.next->self;
+  }
 }
 
 static void pe_timeable_start(pe_event *ev)
 {
   /* OPTIMIZE! */
   pe_tmevent *tm = (pe_tmevent*) ev;
-  pe_ring *rg = Timers.next;
+  pe_ring *rg = Timeables.next;
   while (rg->self && ((pe_tmevent*)rg->self)->tm.at < tm->tm.at) {
     rg = rg->next;
   }
+  /*warn("-- adding 0x%x:\n", ev); db_show_timeables();/**/
   PE_RING_ADD_BEFORE(&tm->tm.ring, rg);
+  /*warn("T:\n"); db_show_timeables();/**/
 }
 
 static void pe_timeable_stop(pe_event *ev)
@@ -45,5 +57,5 @@ static void pe_timeable_stop(pe_event *ev)
 
 void static boot_timeable()
 {
-  PE_RING_INIT(&Timers, 0);
+  PE_RING_INIT(&Timeables, 0);
 }
